namespace dk {
	template <typename T, uint32_t D, typename T1>
	TileIterator<T, D, T1>::TileIterator (qualif_vector_type* parData, const PixelConv<D>& parPixConv, const coords& parArea) :
		m_tile_range(parArea),
		m_data(parData),
		m_pixel_conv(parPixConv)
	{
		DK_ASSERT(parData);
		DK_ASSERT(&m_pixel_conv != nullptr);
	}

	template <typename T, uint32_t D, typename T1>
	TileIterator<T, D, T1>::TileIterator (qualif_vector_type* parData, const PixelConv<D>& parPixConv, const coords& parStart, const coords& parArea) :
		m_tile_range(parStart, parArea),
		m_data(parData),
		m_pixel_conv(parPixConv)
	{
		DK_ASSERT(parData);
		DK_ASSERT(&m_pixel_conv != nullptr);
	}

	template <typename T, uint32_t D, typename T1>
	void TileIterator<T, D, T1>::increment() {
		++m_tile_range;
	}

	template <typename T, uint32_t D, typename T1>
	void TileIterator<T, D, T1>::decrement() {
		--m_tile_range;
	}

	template <typename T, uint32_t D, typename T1>
	void TileIterator<T, D, T1>::advance (size_t /*parAdvance*/) {
		//TODO: implement
	}

	template <typename T, uint32_t D, typename T1>
	auto TileIterator<T, D, T1>::distance_to (const TileIterator& parOther) -> difference_type {
		return std::distance(to_index(m_tile_range), to_index(parOther.m_tile_range));
	}

	template <typename T, uint32_t D, typename T1>
	bool TileIterator<T, D, T1>::equal (const TileIterator& parOther) const {
		return m_data == parOther.m_data and m_tile_range.position() == parOther.m_tile_range.position();
	}

	template <typename T, uint32_t D, typename T1>
	auto TileIterator<T, D, T1>::dereference() const -> reference {
		const auto index = to_index(m_tile_range);
		DK_ASSERT(m_data);
		DK_ASSERT(index >= 0);
		DK_ASSERT(static_cast<typename std::make_unsigned<decltype(index)>::type>(index) < m_data->size());
		return Tile<T, D>(&(*m_data)[index], m_tile_range, coords(0), &m_pixel_conv);
	}

	template <uint32_t D>
	inline Vector<D> make_past_end_coords (const Vector<D>& parTo) {
		Vector<D> retval(0);
		//for (CoordinateDistType d = 0; d < D - 1; ++d) {
			//retval[d] = parFrom[d];
		//}
		retval[D - 1] = parTo[D - 1] + 1;
		return retval;
	}
} //namespace dk
